from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS, parse_BORN
import numpy as np

def append_band(bands, q_start, q_end):
    band = []
    for i in range(51):
        band.append(np.array(q_start) +
                    (np.array(q_end) - np.array(q_start)) / 50 * i)
    bands.append(band)

bulk = read_vasp("POSCAR")
phonon = Phonopy(bulk,
                 [[2, 0, 0],
                  [0, 2, 0],
                  [0, 0, 2]],
                 primitive_matrix=[[0, 0.5, 0.5],
                                   [0.5, 0, 0.5],
                                   [0.5, 0.5, 0]],
                 is_auto_displacements=False)

symmetry = phonon.get_symmetry()
print "Space group:", symmetry.get_international_table()

force_sets = parse_FORCE_SETS()
phonon.set_displacement_dataset(force_sets)
phonon.produce_force_constants()

born = [[[1.08703, 0, 0],
         [0, 1.08703, 0],
         [0, 0, 1.08703]],
        [[-1.08672, 0, 0],
         [0, -1.08672, 0],
         [0, 0, -1.08672]]]
epsilon = [[2.43533967, 0, 0],
           [0, 2.43533967, 0],
           [0, 0, 2.43533967]]
factors = 14.400
phonon.set_nac_params({'born': born,
                       'factor': factors,
                       'dielectric': epsilon})

# BAND = 0.0 0.0 0.0  0.5 0.0 0.0  0.5 0.5 0.0  0.0 0.0 0.0  0.5 0.5 0.5
bands = []
append_band(bands, [0.0, 0.0, 0.0], [0.5, 0.0, 0.0])
append_band(bands, [0.5, 0.0, 0.0], [0.5, 0.5, 0.0])
append_band(bands, [0.5, 0.5, 0.0], [0.0, 0.0, 0.0])
append_band(bands, [0.0, 0.0, 0.0], [0.5, 0.5, 0.5])
phonon.set_band_structure(bands)
q_points, distances, frequencies, eigvecs = phonon.get_band_structure()
for q, d, freq in zip(q_points, distances, frequencies):
    print q, d, freq
phonon.plot_band_structure().show()

# Mesh sampling 20x20x20
phonon.set_mesh([20, 20, 20])
phonon.set_thermal_properties(t_step=10,
                              t_max=1000,
                              t_min=0)

# DOS
phonon.set_total_DOS(sigma=0.1)
for omega, dos in np.array(phonon.get_total_DOS()).T:
    print "%15.7f%15.7f" % (omega, dos)
phonon.plot_total_DOS().show()

# Thermal properties
for t, free_energy, entropy, cv in np.array(phonon.get_thermal_properties()).T:
    print ("%12.3f " + "%15.7f" * 3) % ( t, free_energy, entropy, cv )
phonon.plot_thermal_properties().show()

# PDOS
phonon.set_mesh([20, 20, 20],
                is_mesh_symmetry=False,
                is_eigenvectors=True)
phonon.set_partial_DOS(tetrahedron_method=True)
omegas, pdos = phonon.get_partial_DOS()
pdos_indices = [[0], [1]]
phonon.plot_partial_DOS(pdos_indices=pdos_indices,
                        legend=pdos_indices).show()



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np
from phonopy.structure.symmetry import Symmetry, get_pointgroup
from phonopy.harmonic.force_constants import similarity_transformation
from phonopy.phonon.degeneracy import degenerate_sets as get_degenerate_sets
from phonopy.units import VaspToTHz
from phonopy.harmonic.derivative_dynmat import DerivativeOfDynamicalMatrix

# from Wikipedia http://en.wikipedia.org/wiki/List_of_character_tables_for_chemically_important_3D_point_groups
character_table = { 
    '6/mmm':
    { 'rotation_list':
      [ 'E', 'C6', 'C3', 'C2', 'C2\'', 'C2\'\'',
        'i', 'S3', 'S6', 'sgh', 'sgd', 'sgv' ],
      'character_table':
      { 'A1g': [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 ],
        'A2g': [ 1, 1, 1, 1,-1,-1, 1, 1, 1, 1,-1,-1 ],
        'B1g': [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1, 1,-1 ],
        'B2g': [ 1,-1, 1,-1,-1, 1, 1,-1, 1,-1,-1, 1 ],
        'E1g': [ 2, 1,-1,-2, 0, 0, 2, 1,-1,-2, 0, 0 ],
        'E2g': [ 2,-1,-1, 2, 0, 0, 2,-1,-1, 2, 0, 0 ],
        'A1u': [ 1, 1, 1, 1, 1, 1,-1,-1,-1,-1,-1,-1 ],
        'A2u': [ 1, 1, 1, 1,-1,-1,-1,-1,-1,-1, 1, 1 ],
        'B1u': [ 1,-1, 1,-1, 1,-1,-1, 1,-1, 1,-1, 1 ],
        'B2u': [ 1,-1, 1,-1,-1, 1,-1, 1,-1, 1, 1,-1 ],
        'E1u': [ 2, 1,-1,-2, 0, 0,-2,-1, 1, 2, 0, 0 ],
        'E2u': [ 2,-1,-1, 2, 0, 0,-2, 1, 1,-2, 0, 0 ] },
      'mapping_table': [
          { 'E'     : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C6'    : [ ( ( 1,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 0, 1, 0 ),
                          (-1, 1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C3'    : [ ( ( 0,-1, 0 ),
                          ( 1,-1, 0 ),
                          ( 0, 0, 1 ) ),
                        ( (-1, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2'    : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2\''  : [ ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( (-1, 1, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 1, 0, 0 ),
                          ( 1,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'C2\'\'': [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 1,-1, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ),
                        ( (-1, 0, 0 ),
                          (-1, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'i'     : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'S3'    : [ ( (-1, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 0,-1, 0 ),
                          ( 1,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'S6'    : [ ( ( 0, 1, 0 ),
                          (-1, 1, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 1,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgh'   : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgd'   : [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 1,-1, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ),
                        ( (-1, 0, 0 ),
                          (-1, 1, 0 ),
                          ( 0, 0, 1 ) ) ],             
            'sgv'   : [ ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( (-1, 1, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 1, 0, 0 ),
                          ( 1,-1, 0 ),
                          ( 0, 0, 1 ) ) ] }
      ]
    },

    '6mm':
    { 'rotation_list':
      [ 'E', 'C6', 'C3', 'C2', 'sgv', 'sgd' ],
      'character_table':
      { 'A1': [ 1, 1, 1, 1, 1, 1 ],
        'A2': [ 1, 1, 1, 1,-1,-1 ],
        'B1': [ 1,-1, 1,-1, 1,-1 ],
        'B2': [ 1,-1, 1,-1,-1, 1 ],
        'E1': [ 2, 1,-1,-2, 0, 0 ],
        'E2': [ 2,-1,-1, 2, 0, 0 ] },
      'mapping_table': [
          { 'E'  : [ ( ( 1, 0, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C6' : [ ( ( 1,-1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0, 1 ) ),
                     ( ( 0, 1, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C2' : [ ( (-1, 0, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'sgv': [ ( ( 0,-1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( ( 1, 0, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'sgd': [ ( ( 0, 1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0, 1 ) ),
                     ( ( 1,-1, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 0, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0, 1 ) ) ] }
      ]
    },

    '4/mmm':
    { 'rotation_list':
      [ 'E', 'C4', 'C2', 'C2\'', 'C2\'\'', 'i',
        'S4', 'sgh', 'sgv', 'sgd' ],
      'character_table':
      { 'A1g': [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 ],
        'A2g': [ 1, 1, 1,-1,-1, 1, 1, 1,-1,-1 ],
        'B1g': [ 1,-1, 1, 1,-1, 1,-1, 1, 1,-1 ],
        'B2g': [ 1,-1, 1,-1, 1, 1,-1, 1,-1, 1 ],
        'Eg' : [ 2, 0,-2, 0, 0, 2, 0,-2, 0, 0 ],
        'A1u': [ 1, 1, 1, 1, 1,-1,-1,-1,-1,-1 ],
        'A2u': [ 1, 1, 1,-1,-1,-1,-1,-1, 1, 1 ],
        'B1u': [ 1,-1, 1, 1,-1,-1, 1,-1,-1, 1 ],
        'B2u': [ 1,-1, 1,-1, 1,-1, 1,-1, 1,-1 ],
        'Eu' : [ 2, 0,-2, 0, 0,-2, 0, 2, 0, 0 ] },
      'mapping_table': [
          { 'E'     : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C4'    : [ ( ( 0,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 0, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2'    : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2\''  : [ ( ( 1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ),
                        ( (-1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'C2\'\'': [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ) ],
            'i'     : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'S4'    : [ ( ( 0, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 0,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgh'   : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgv'   : [ ( (-1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'sgd'   : [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ) ] }
      ]
    },

    '422':
    { 'rotation_list':
      [ 'E', 'C4', 'C2', 'C2\'', 'C2\'\''],
      'character_table':
      { 'A1': [ 1, 1, 1, 1, 1 ],
        'A2': [ 1, 1, 1,-1,-1 ],
        'B1': [ 1,-1, 1, 1,-1 ],
        'B2': [ 1,-1, 1,-1, 1 ],
        'E' : [ 2, 0,-2, 0, 0 ] },
      'mapping_table': [
          { 'E'     : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C4'    : [ ( ( 0,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 0, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2'    : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2\''  : [ ( ( 1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ),
                        ( (-1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'C2\'\'': [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ) ],
            'i'     : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'S4'    : [ ( ( 0, 1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0,-1 ) ),
                        ( ( 0,-1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgh'   : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgv'   : [ ( (-1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'sgd'   : [ ( ( 0, 1, 0 ),
                          ( 1, 0, 0 ),
                          ( 0, 0, 1 ) ),
                        ( ( 0,-1, 0 ),
                          (-1, 0, 0 ),
                          ( 0, 0, 1 ) ) ] }
      ]
    },

    'mmm':
    { 'rotation_list':
      [ 'E', 'C2', 'C2x', 'C2y', 'i', 'sgxy',
        'sgxz', 'sgyz' ],
      'character_table':
      { 'Ag' : [ 1, 1, 1, 1, 1, 1, 1, 1 ],
        'B1g': [ 1, 1,-1,-1, 1, 1,-1,-1 ],
        'B2g': [ 1,-1,-1, 1, 1,-1, 1,-1 ],
        'B3g': [ 1,-1, 1,-1, 1,-1,-1, 1 ],
        'Au' : [ 1, 1, 1, 1,-1,-1,-1,-1 ],
        'B1u': [ 1, 1,-1,-1,-1,-1, 1, 1 ],
        'B2u': [ 1,-1,-1, 1,-1, 1,-1, 1 ],
        'B3u': [ 1,-1, 1,-1,-1, 1, 1,-1 ] },
      'mapping_table': [
          { 'E'   : [ ( ( 1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'C2'  : [ ( (-1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'C2y' : [ ( (-1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'C2x' : [ ( ( 1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'i'   : [ ( (-1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'sgxy': [ ( ( 1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'sgxz': [ ( ( 1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'sgyz': [ ( (-1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0, 1 ) ) ] }
      ]
    },

    'm-3m':
    { 'rotation_list':
      [ 'E', 'C3', 'C2', 'C4', 'C4^2', 'i',
        'S4', 'S6', 'sgh', 'sgd' ],
      'character_table':
      { 'A1g': [ 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 ],
        'A2g': [ 1, 1,-1,-1, 1, 1,-1, 1, 1,-1 ],
        'Eg' : [ 2,-1, 0, 0, 2, 2, 0,-1, 2, 0 ],
        'T1g': [ 3, 0,-1, 1,-1, 3, 1, 0,-1,-1 ],
        'T2g': [ 3, 0, 1,-1,-1, 3,-1, 0,-1, 1 ],
        'A1u': [ 1, 1, 1, 1, 1,-1,-1,-1,-1,-1 ],
        'A2u': [ 1, 1,-1,-1, 1,-1, 1,-1,-1, 1 ],
        'Eu' : [ 2,-1, 0, 0, 2,-2, 0, 1,-2, 0 ],
        'T1u': [ 3, 0,-1, 1,-1,-3,-1, 0, 1, 1 ],
        'T2u': [ 3, 0, 1,-1,-1,-3, 1, 0, 1,-1 ] },
      'mapping_table': [
          { 'E'   : [ ( ( 1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'C3'  : [ ( ( 0, 0, 1 ),
                        ( 1, 0, 0 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 1, 0 ),
                        ( 0, 0, 1 ),
                        ( 1, 0, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 1, 0, 0 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0,-1, 0 ),
                        ( 0, 0,-1 ),
                        ( 1, 0, 0 ) ),
                      ( ( 0, 0,-1 ),
                        (-1, 0, 0 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 1, 0 ),
                        ( 0, 0,-1 ),
                        (-1, 0, 0 ) ),
                      ( ( 0, 0, 1 ),
                        (-1, 0, 0 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0,-1, 0 ),
                        ( 0, 0, 1 ),
                        (-1, 0, 0 ) ) ],
            'C2'  : [ ( ( 0, 1, 0 ),
                        ( 1, 0, 0 ),
                        ( 0, 0,-1 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 0, 1 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 0, 1 ),
                        ( 0,-1, 0 ),
                        ( 1, 0, 0 ) ),
                      ( ( 0,-1, 0 ),
                        (-1, 0, 0 ),
                        ( 0, 0,-1 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 0,-1 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 0,-1, 0 ),
                        (-1, 0, 0 ) ) ],
            'C4'  : [ ( ( 0,-1, 0 ),
                        ( 1, 0, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 0, 1, 0 ),
                        (-1, 0, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0, 0,-1 ),
                        ( 0, 1, 0 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0, 0, 1 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0, 0, 1 ),
                        ( 0, 1, 0 ),
                        (-1, 0, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 0, 1, 0 ),
                        ( 1, 0, 0 ) ) ],
            'C4^2': [ ( (-1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0,-1 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'i'   : [ ( (-1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0,-1 ) ) ],
            'S4'  : [ ( ( 0, 1, 0 ),
                        (-1, 0, 0 ),
                        ( 0, 0,-1 ) ),
                      ( ( 0,-1, 0 ),
                        ( 1, 0, 0 ),
                        ( 0, 0,-1 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 0, 1 ),
                        ( 0,-1, 0 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 0,-1 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 0,-1, 0 ),
                        ( 1, 0, 0 ) ),
                      ( ( 0, 0, 1 ),
                        ( 0,-1, 0 ),
                        (-1, 0, 0 ) ) ],
            'S6'  : [ ( ( 0, 0,-1 ),
                        (-1, 0, 0 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0,-1, 0 ),
                        ( 0, 0,-1 ),
                        (-1, 0, 0 ) ),
                      ( ( 0, 0, 1 ),
                        (-1, 0, 0 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 1, 0 ),
                        ( 0, 0, 1 ),
                        (-1, 0, 0 ) ),
                      ( ( 0, 0, 1 ),
                        ( 1, 0, 0 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0,-1, 0 ),
                        ( 0, 0, 1 ),
                        ( 1, 0, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 1, 0, 0 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 1, 0 ),
                        ( 0, 0,-1 ),
                        ( 1, 0, 0 ) ) ],
            'sgh' : [ ( ( 1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0,-1 ) ),
                      ( (-1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0,-1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'sgd' : [ ( ( 0,-1, 0 ),
                        (-1, 0, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0, 0,-1 ),
                        ( 0,-1, 0 ) ),
                      ( ( 0, 0,-1 ),
                        ( 0, 1, 0 ),
                        (-1, 0, 0 ) ),
                      ( ( 0, 1, 0 ),
                        ( 1, 0, 0 ),
                        ( 0, 0, 1 ) ),
                      ( ( 1, 0, 0 ),
                        ( 0, 0, 1 ),
                        ( 0, 1, 0 ) ),
                      ( ( 0, 0, 1 ),
                        ( 0, 1, 0 ),
                        ( 1, 0, 0 ) ) ]
          }
      ]
    },

    '-6m2':
    { 'rotation_list':
      [ 'E', 'C3', 'C2', 'sgh', 'S3', 'sgv' ],
      'character_table':
      { 'A1\''  : [ 1, 1, 1, 1, 1, 1 ],
        'A2\''  : [ 1, 1,-1, 1, 1,-1 ],
        'E\''   : [ 2,-1, 0, 2,-1, 0 ],
        'A1\'\'': [ 1, 1, 1,-1,-1,-1 ],
        'A2\'\'': [ 1, 1,-1,-1,-1, 1 ],
        'E\'\'' : [ 2,-1, 0,-2, 1, 0 ] },
      'mapping_table': [
          { 'E'   : [ ( ( 1, 0, 0 ),
                        ( 0, 1, 0 ),
                        ( 0, 0, 1 ) ) ],
            'C3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C2' : [ ( ( 0, 1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0,-1 ) ),
                     ( ( 1,-1, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0,-1 ) ),
                     ( (-1, 0, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0,-1 ) ) ],
            'sgh': [ ( ( 1, 0, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0,-1 ) ) ],
            'S3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0,-1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0,-1 ) ) ],
            'sgv': [ ( ( 0, 1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0, 1 ) ),
                     ( ( 1,-1, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 0, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0, 1 ) ) ]
          }
      ]
    },
    
    '3':
    { 'rotation_list':
      [ 'E', 'C3'],
      'character_table':
      { 'A': [ 1, 1 ],
        'E': [ 2,-1 ] },
      'mapping_table': [
          { 'E'  : [ ( ( 1, 0, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ) ]
          }
      ]
    },

    '-3m':
    { 'rotation_list':
      [ 'E', 'C3', 'C2', 'i', 'S6', 'sgd' ],
      'character_table':
      { 'A1g': [ 1, 1, 1, 1, 1, 1 ],
        'A2g': [ 1, 1,-1, 1, 1,-1 ],
        'Eg' : [ 2,-1, 0, 2,-1, 0 ],
        'A1u': [ 1, 1, 1,-1,-1,-1 ],
        'A2u': [ 1, 1,-1,-1,-1, 1 ],
        'Eu' : [ 2,-1, 0,-2, 1, 0 ] },
      'mapping_table': [
          { 'E'  : [ ( ( 1, 0, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C2' : [ ( ( 0,-1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0,-1 ) ),
                     ( (-1, 1, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0,-1 ) ),
                     ( ( 1, 0, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0,-1 ) ) ],
            'i'  : [ ( (-1, 0, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0,-1 ) ) ],
            'S6' : [ ( ( 0, 1, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0,-1 ) ),
                     ( ( 1,-1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0,-1 ) ) ],
            'sgd': [ ( ( 0, 1, 0 ),
                       ( 1, 0, 0 ),
                       ( 0, 0, 1 ) ),
                     ( ( 1,-1, 0 ),
                       ( 0,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 0, 0 ),
                       (-1, 1, 0 ),
                       ( 0, 0, 1 ) ) ]
          }
      ]
    },

    '32':
    { 'rotation_list':
      [ 'E', 'C3', 'C2\'' ],
      'character_table':
      { 'A1': [ 1, 1, 1 ],
        'A2': [ 1, 1,-1 ],
        'E' : [ 2,-1, 0 ] },
      'mapping_table': [
          { 'E'  : [ ( ( 1, 0, 0 ),
                       ( 0, 1, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C3' : [ ( ( 0,-1, 0 ),
                       ( 1,-1, 0 ),
                       ( 0, 0, 1 ) ),
                     ( (-1, 1, 0 ),
                       (-1, 0, 0 ),
                       ( 0, 0, 1 ) ) ],
            'C2\'' : [ ( ( 0, 1, 0 ),
                         ( 1, 0, 0 ),
                         ( 0, 0,-1 ) ),
                       ( ( 1,-1, 0 ),
                         ( 0,-1, 0 ),
                         ( 0, 0,-1 ) ),
                       ( (-1, 0, 0 ),
                         (-1, 1, 0 ),
                         ( 0, 0,-1 ) ) ]
          }
      ]
    },

    '2/m':
    { 'rotation_list':
      [ 'E', 'C2', 'i', 'sgh'],
      'character_table':
      {'Ag': [1, 1, 1, 1],
       'Bg': [1, -1, 1, -1],
       'Au': [1, 1, -1, -1],
       'Bu': [1, -1, -1, 1] },
      'mapping_table': [
          { 'E'     : [ ( ( 1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0, 1 ) ) ],
            'C2'    : [ ( (-1, 0, 0 ),
                          ( 0, 1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'i'     : [ ( (-1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0,-1 ) ) ],
            'sgh'   : [ ( ( 1, 0, 0 ),
                          ( 0,-1, 0 ),
                          ( 0, 0, 1 ) ) ] }
      ]
    }
}


class IrReps:
    def __init__(self,
                 dynamical_matrix,
                 q,
                 is_little_cogroup=False,
                 nac_q_direction=None,
                 factor=VaspToTHz,
                 symprec=1e-5,
                 degeneracy_tolerance=1e-5,
                 log_level=0):
        self._is_little_cogroup = is_little_cogroup
        self._nac_q_direction = nac_q_direction
        self._factor = factor
        self._log_level = log_level

        self._q = np.array(q)
        self._degeneracy_tolerance = degeneracy_tolerance
        self._symprec = symprec
        self._primitive = dynamical_matrix.get_primitive()
        self._dynamical_matrix = dynamical_matrix
        self._ddm = DerivativeOfDynamicalMatrix(dynamical_matrix)

    def run(self):
        self._set_eigenvectors(self._dynamical_matrix)
        self._symmetry_dataset = Symmetry(self._primitive,
                                          self._symprec).get_dataset()

        if not self._is_primitive_cell():
            print('')
            print("Non-primitve cell is used.")
            print("Your unit cell may be transformed to a primitive cell "
                  "by PRIMITIVE_AXIS tag.")
            return False
        
        (self._rotations_at_q,
         self._translations_at_q) = self._get_rotations_at_q()

        self._g = len(self._rotations_at_q)

        (self._pointgroup_symbol,
         self._transformation_matrix,
         self._conventional_rotations) = self._get_conventional_rotations()

        self._ground_matrices = self._get_ground_matrix()
        self._degenerate_sets = self._get_degenerate_sets()
        self._irreps = self._get_irreps()
        self._characters, self._irrep_dims = self._get_characters()

        self._ir_labels = None
        if self._pointgroup_symbol in character_table.keys():
            self._rotation_symbols = self._get_rotation_symbols()
            if (abs(self._q) < self._symprec).all() and self._rotation_symbols:
                self._ir_labels = self._get_ir_labels()
            elif (abs(self._q) < self._symprec).all():
                if self._log_level > 0:
                    print("Database for this point group is not preprared.")
            else:
                if self._log_level > 0:
                    print("Database for non-Gamma point is not prepared.")
        else:
            self._rotation_symbols = None

        return True

    def _get_degenerate_sets(self):
        deg_sets = get_degenerate_sets(self._freqs,
                                       cutoff=self._degeneracy_tolerance)
        self._ddm.run(self._q)
        ddm_q = np.sum([self._ddm.get_derivative_of_dynamical_matrix()[i] *
                        self._q[i] for i in range(3)], axis=0)
        return deg_sets
        
    def get_band_indices(self):
        return self._degenerate_sets

    def get_characters(self):
        return self._characters

    def get_eigenvectors(self):
        return self._eigvecs

    def get_irreps(self):
        return self._irreps

    def get_ground_matrices(self):
        return self._ground_matrices

    def get_rotation_symbols(self):
        return self._rotation_symbols

    def get_rotations(self):
        return self._conventional_rotations

    def get_projection_operators(self, idx_irrep, i=None, j=None):
        if i is None or j is None:
            return self._get_character_projection_operators(idx_irrep)
        else:
            return self._get_projection_operators(idx_irrep, i, j)

    def show(self, show_irreps=False):
        self._show(show_irreps)

    def write_yaml(self, show_irreps=False):
        self._write_yaml(show_irreps)

    def _set_eigenvectors(self, dm):
        if self._nac_q_direction is not None and (np.abs(self._q) < 1e-5).all():
            dm.set_dynamical_matrix(self._q, q_direction=self._nac_q_direction)
        else:        
            dm.set_dynamical_matrix(self._q)
        eigvals, self._eigvecs = np.linalg.eigh(dm.get_dynamical_matrix())
        self._freqs = np.sqrt(abs(eigvals)) * np.sign(eigvals) * self._factor

    def _get_rotations_at_q(self):
        rotations_at_q = []
        trans_at_q = []
        for r, t in zip(self._symmetry_dataset['rotations'],
                        self._symmetry_dataset['translations']):

            # Using r is used instead of np.linalg.inv(r)
            diff = np.dot(self._q, r) - self._q 

            if (abs(diff - np.rint(diff)) < self._symprec).all():
                rotations_at_q.append(r)
                for i in range(3):
                    if np.abs(t[i] - 1) < self._symprec:
                        t[i] = 0.0
                trans_at_q.append(t)

        return np.array(rotations_at_q), np.array(trans_at_q)

    def _get_conventional_rotations(self):
        spacegroup_symbol = self._symmetry_dataset['international'][0]
        spacegroup_number = self._symmetry_dataset['number']
        rotations = self._rotations_at_q.copy()
        pointgroup = get_pointgroup(rotations)
        pointgroup_symbol = pointgroup[0]
        transformation_matrix = pointgroup[1]

        conventional_rotations = self._transform_rotations(
            transformation_matrix, rotations)

        return (pointgroup_symbol,
                transformation_matrix,
                conventional_rotations)

    def _transform_rotations(self, tmat, rotations):
        trans_rots = []

        for r in rotations:
            r_conv = similarity_transformation(np.linalg.inv(tmat), r)
            trans_rots.append(np.rint(r_conv).astype(int))

        return np.array(trans_rots)

    def _get_ground_matrix(self):
        matrices = []
        
        for (r, t) in zip(self._rotations_at_q,
                          self._translations_at_q):
    
            lat = self._primitive.get_cell().T
            r_cart = similarity_transformation(lat, r)
    
            perm_mat = self._get_modified_permutation_matrix(r, t)
            matrices.append(np.kron(perm_mat, r_cart))

        return np.array(matrices)

    def _get_characters(self):
        characters = []
        irrep_dims = []
        for irrep_Rs in self._irreps:
            characters.append([np.trace(rep) for rep in irrep_Rs])
            irrep_dims.append(len(irrep_Rs[0]))
        return np.array(characters), np.array(irrep_dims)

    def _get_modified_permutation_matrix(self, r, t, ):
        num_atom = self._primitive.get_number_of_atoms()
        pos = self._primitive.get_scaled_positions()
        matrix = np.zeros((num_atom, num_atom), dtype=complex)
        for i, p1 in enumerate(pos):
            p_rot = np.dot(r, p1) + t
            for j, p2 in enumerate(pos):
                diff = p_rot - p2
                if (abs(diff - np.rint(diff)) < self._symprec).all():
                    # For this phase factor, see
                    # Dynamics of perfect crystals by G. Venkataraman et al.,
                    # pp132 Eq. (3.22).
                    # It is assumed that dynamical matrix is built without
                    # considering internal atomic positions, so
                    # the phase factors of eigenvectors are shifted in
                    # _get_irreps().
                    phase_factor = np.dot(
                        self._q, np.dot(np.linalg.inv(r), p2 - p_rot))
                    
                    # This phase factor comes from non-pure-translation of
                    # each symmetry opration.
                    if self._is_little_cogroup:
                        phase_factor += np.dot(t, self._q)
                        
                    matrix[j, i] = np.exp(2j * np.pi * phase_factor)

        return matrix
    
    def _get_irreps(self):
        eigvecs = []
        phases = np.kron(
            [np.exp(2j * np.pi * np.dot(self._q, pos))
             for pos in self._primitive.get_scaled_positions()], [1, 1, 1])
        for vec in self._eigvecs.T:
            eigvecs.append(vec * phases)

        irrep = []
        for band_indices in self._degenerate_sets:
            irrep_Rs = []
            for mat in self._ground_matrices:
                l = len(band_indices)

                if l == 1:
                    vec = eigvecs[band_indices[0]]
                    irrep_Rs.append([[np.vdot(vec, np.dot(mat, vec))]])
                    continue
                
                irrep_R = np.zeros((l, l), dtype=complex)
                for i, b_i in enumerate(band_indices):
                    vec_i = eigvecs[b_i]
                    for j, b_j in enumerate(band_indices):
                        vec_j = eigvecs[b_j]
                        irrep_R[i, j] = np.vdot(vec_i, np.dot(mat, vec_j))
                irrep_Rs.append(irrep_R)

            irrep.append(irrep_Rs)

        return irrep

    def _get_character_projection_operators(self, idx_irrep):
        dim = self._irrep_dims[idx_irrep]
        chars = self._characters[idx_irrep]
        return np.sum([mat * char.conj()
                       for mat, char in zip(self._ground_matrices, chars)],
                      axis=0) * dim / self._g

    def _get_projection_operators(self, idx_irrep, i, j):
        dim = self._irrep_dims[idx_irrep]
        return np.sum([mat * r[i, j].conj() for mat, r
                       in zip(self._ground_matrices, self._irreps[idx_irrep])],
                      axis=0) * dim / self._g
    
    def _get_rotation_symbols(self):
        ptg = self._pointgroup_symbol
        for mapping_table in character_table[ptg]['mapping_table']:
            rotation_symbols = []
            for r in self._conventional_rotations:
                symbol = get_rotation_symbol(r, mapping_table)
                rotation_symbols.append(symbol)

            if not False in rotation_symbols:
                break

        if False in rotation_symbols:
            return None
        else:
            return rotation_symbols

    def _get_ir_labels(self):
        ir_labels = []
        rot_list = character_table[self._pointgroup_symbol]['rotation_list']
        char_table = character_table[self._pointgroup_symbol]['character_table']
        for chars, deg_set in zip(self._characters, self._degenerate_sets):
            chars_ordered = np.zeros(len(rot_list), dtype=complex)
            for rs, ch in zip(self._rotation_symbols, chars):
                chars_ordered[rot_list.index(rs)] += ch

            for i, rl in enumerate(rot_list):
                chars_ordered[i] /= len(
                    character_table[self._pointgroup_symbol]['mapping_table'][0][rl])

            found = False
            for ct_label in char_table.keys():
                if (abs(chars_ordered - np.array(char_table[ct_label])) <
                    self._symprec).all():
                    ir_labels.append(ct_label)
                    found = True
                    break
                
            if not found:
                ir_labels.append(None)

            if self._log_level > 1:
                text = ""
                for v in chars_ordered:
                    text += "%5.2f " % abs(v)
                if found:
                    print("%s %s" % (text, ct_label))
                else:
                    print("%s Not found" % text)
                
        return ir_labels

    def _is_primitive_cell(self):
        num_identity = 0
        for r in self._symmetry_dataset['rotations']:
            if (r - np.eye(3, dtype='intc') == 0).all():
                num_identity += 1
                if num_identity > 1:
                    return False
        else:
            return True
    
    def _show(self, show_irreps):
        print('')
        print("-------------------------------")
        print("  Irreducible representations")
        print("-------------------------------")
        print("q-point: %s" % self._q)
        print("Point group: %s" % self._pointgroup_symbol)
        print('')
        
        if (np.abs(self._q) < self._symprec).all():
            width = 6
            print("Original rotation matrices:")
            print('')
            print_rotations(self._rotations_at_q, width=width)
        else:
            width = 4
            print("Original symmetry operations:")
            print('')
            print_rotations(self._rotations_at_q,
                            translations=self._translations_at_q,
                            width=width)

        print("Transformation matrix:")
        print('')
        for v in self._transformation_matrix:
            print("%6.3f %6.3f %6.3f" % tuple(v))
        print('')
        print("Rotation matrices by transformation matrix:")
        print('')
        print_rotations(self._conventional_rotations,
                        rotation_symbols=self._rotation_symbols,
                        width=width)
        print("Character table:")
        print('')
        for i, deg_set in enumerate(self._degenerate_sets):
            text = "%3d (%8.3f): " % (deg_set[0] + 1, self._freqs[deg_set[0]])
            if self._ir_labels is None:
                print(text)
            else:
                print(text + self._ir_labels[i])
            print_characters(self._characters[i])
            print('')

        if show_irreps:
            self._show_irreps()
    
    def _show_irreps(self):
        print("IR representations:")
        print('')
        
        for i, (deg_set, irrep_Rs) in enumerate(zip(self._degenerate_sets,
                                                    self._irreps)):
            print("%3d (%8.3f):" % (deg_set[0] + 1, self._freqs[deg_set[0]]))
            print('')
            for j, irrep_R in enumerate(irrep_Rs):
                for k in range(len(irrep_R)):
                    text = "     "
                    for l in range(len(irrep_R)):
                        if irrep_R[k][l].real > 0:
                            sign_r = " "
                        else:
                            sign_r = "-"
                        
                        if irrep_R[k][l].imag > 0:
                            sign_i = "+"
                        else:
                            sign_i = "-"

                        if k == 0:
                            str_index = "%2d" % (j + 1)
                        else:
                            str_index = "  "
                            
                        if l > 0:
                            str_index = ''

                        text += "%s (%s%5.3f %s%5.3fi) " % (
                            str_index,
                            sign_r, abs(irrep_R[k][l].real),
                            sign_i, abs(irrep_R[k][l].imag))
                    print(text)
                if len(irrep_R) > 1:
                    print('')
            if len(irrep_R) == 1:
                print('')

    def _write_yaml(self, show_irreps):
        w = open("irreps.yaml", 'w')
        w.write("q-position: [ %12.7f, %12.7f, %12.7f ]\n" % tuple(self._q))
        w.write("point_group: %s\n" % self._pointgroup_symbol)
        w.write("transformation_matrix:\n")
        for v in self._transformation_matrix:
            w.write("- [ %10.7f, %10.7f, %10.7f ]\n" % tuple(v))
        w.write("rotations:\n")
        for i, r in enumerate(self._conventional_rotations):
            w.write("- matrix:\n")
            for v in r:
                w.write("  - [ %2d, %2d, %2d ]\n" % tuple(v))
            if self._rotation_symbols:
                w.write("  symbol: %s\n" % self._rotation_symbols[i])
        w.write("normal_modes:\n")
        for i, deg_set in enumerate(self._degenerate_sets):
            w.write("- band_indices: [ ")
            w.write("%d" % (deg_set[0] + 1))
            for bi in deg_set[1:]:
                w.write(", %d" % (bi + 1))
            w.write(" ]\n")
            w.write("  frequency: %-15.10f\n" % self._freqs[deg_set[0]])
            if self._ir_labels:
                w.write("  ir_label: %s\n" % self._ir_labels[i])
            w.write("  characters: ")
            chars = np.rint(np.abs(self._characters[i]))
            phase = (np.angle(self._characters[i]) / np.pi * 180) % 360
            if len(chars) > 1:
                w.write("[ [ %2d, %5.1f ]" % (chars[0], phase[0]))
                for chi, theta in zip(chars[1:], phase[1:]):
                    w.write(", [ %2d, %5.1f ]" % (chi, theta))
                w.write(" ]\n")
            else:
                w.write("[ [ %2d, %5.1f ] ]\n" % (chars[0], phase[0]))

        if show_irreps:
            self._write_yaml_irreps(w)
            
        w.close()

    def _write_yaml_irreps(self, file_pointer):
        w = file_pointer
        if not self._irreps:
            self._irrep = self._get_irreps()

        w.write("\n")
        w.write("irreps:\n")
        for i, (deg_set, irrep_Rs) in enumerate(
            zip(self._degenerate_sets, self._irreps)):
            w.write("- # %d\n" % (i + 1))
            for j, irrep_R in enumerate(irrep_Rs):
                if self._rotation_symbols:
                    symbol = self._rotation_symbols[j]
                else:
                    symbol = ''
                if len(deg_set) > 1:
                    w.write("  - # %d %s\n" % (j + 1, symbol))
                    for k, v in enumerate(irrep_R):
                        w.write("    - [ ")
                        for x in v[:-1]:
                            w.write("%10.7f, %10.7f,   " % (x.real, x.imag))
                        w.write("%10.7f, %10.7f ] # (" %
                                (v[-1].real, v[-1].imag))
                        
                        w.write(("%5.0f" * len(v)) %
                                tuple((np.angle(v) / np.pi * 180) % 360))
                        w.write(")\n")
                else:
                    x = irrep_R[0][0]
                    w.write("  - [ [ %10.7f, %10.7f ] ] # (%3.0f) %d %s\n" %
                            (x.real, x.imag,
                             (np.angle(x) / np.pi * 180) % 360, j + 1, symbol))
                
        pass
                
def get_rotation_symbol(rotation, mapping_table):
    for k, v in mapping_table.iteritems():
        for r in v:
            if (r == rotation).all():
                return k
    return False

def print_characters(characters, width=6):
    text = ""
    for i, c in enumerate(characters):
        angle = np.angle(c) / np.pi * 180
        if angle < 0:
            angle += 360
        angle = np.around(angle, decimals=2) % 360
        val = abs(c)
        if val < 1e-5:
            val = 0
            angle = 0
        else:
            val = np.rint(val)
        text += "(%2d, %5.1f) " % (val, angle)
        if ((i + 1) % width == 0 and i != 0) or (len(characters) == i + 1):
            print("    " + text)
            text = ""

def print_rotations(rotations,
                    translations=None,
                    rotation_symbols=None,
                    width=6):
    for i in range(len(rotations) // width):
        if rotation_symbols is None:
            if translations is None:
                print(("    %2d    " * width) %
                      tuple(np.arange(i * width, (i + 1) * width) + 1))
            else:
                print(("       %2d       " * width) %
                      tuple(np.arange(i * width, (i + 1) * width) + 1))
        else:
            text = ""
            for k in range(width):
                rot_symbol = rotation_symbols[i * width + k]
                if translations is None:
                    if len(rot_symbol) < 3:
                        text += "    %2s    " % rot_symbol
                    else:
                        text += "   %4s   " % rot_symbol
                else:
                    if len(rot_symbol) < 3:
                        text += "       %2s        " % rot_symbol
                    else:
                        text += "     %4s      " % rot_symbol
            print(text)
        if translations is None:
            print(" -------- " * width)
        else:
            print(" -------------- " * width)

        for j in range(3):
            text = ""
            for k in range(width):
                text += " %2d %2d %2d " % tuple(rotations[i * width + k][j])
                if translations is not None:
                    text += "%5.2f " % translations[i * width + k][j]
            print(text)
        print('')
    
    num_rest = len(rotations) % width
    if num_rest > 0:
        i = len(rotations) // width
        if rotation_symbols is None:
            print(("    %2d    " * num_rest) %
                  tuple(np.arange(i * width, i * width + num_rest) + 1))
        else:
            text = ""
            for k in range(num_rest):
                rot_symbol = rotation_symbols[i * width + k]
                if len(rot_symbol) < 3:
                    text += "    %2s    " % rot_symbol
                else:
                    text += "   %4s   " % rot_symbol
            print(text)
        print(" -------- " * num_rest)
        for j in range(3):
            text = ""
            for k in range(num_rest):
                text += (" %2d %2d %2d ") % tuple(rotations[i * width + k][j])
            print(text)
        print('')




from gpaw import GPAW, PW
from ase import *
from phonopy import Phonopy
from phonopy.structure.atoms import Atoms as PhonopyAtoms
import numpy as np

# GPAW setting
a = 5.404
bulk = PhonopyAtoms(symbols=['Si']*8,
                    scaled_positions=[(0, 0, 0),
                                      (0, 0.5, 0.5),
                                      (0.5, 0, 0.5),
                                      (0.5, 0.5, 0),
                                      (0.25, 0.25, 0.25),
                                      (0.25, 0.75, 0.75),
                                      (0.75, 0.25, 0.75),
                                      (0.75, 0.75, 0.25)] )
bulk.set_cell(np.diag((a, a, a)))

calc = GPAW(mode=PW(300),
            kpts={'size': (4, 4, 4)},
            symmetry={'symmorphic': False})

phonon = Phonopy(bulk,
                 [[1,0,0],[0,1,0],[0,0,1]],
                 primitive_matrix=[[0, 0.5, 0.5],
                                   [0.5, 0, 0.5],
                                   [0.5, 0.5, 0]],
                 distance=0.01)
print "[Phonopy] Atomic displacements:"
disps = phonon.get_displacements()
for d in disps:
    print "[Phonopy]", d[0], d[1:]
supercells = phonon.get_supercells_with_displacements()

# Force calculations by calculator
set_of_forces = []
for scell in supercells:
    cell = Atoms(symbols=scell.get_chemical_symbols(),
                 scaled_positions=scell.get_scaled_positions(),
                 cell=scell.get_cell(),
                 pbc=True)
    cell.set_calculator(calc)
    forces = cell.get_forces()
    drift_force = forces.sum(axis=0)
    print "[Phonopy] Drift force:", "%11.5f"*3 % tuple(drift_force)
    # Simple translational invariance
    for force in forces:
        force -= drift_force / forces.shape[0]
    set_of_forces.append(forces)

# Phonopy post-process
phonon.produce_force_constants(forces=set_of_forces)
print
print "[Phonopy] Phonon frequencies at Gamma:"
for i, freq in enumerate(phonon.get_frequencies((0, 0, 0))):
    print "[Phonopy] %3d: %10.5f THz" %  (i + 1, freq) # THz





# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import numpy as np

class Forces:
    """
    forces: Forces on atoms in a supercell with a displacement in Cartesian coordinate
      [ [ F_1x, F_1y, F_1z ], 
        [ F_2x, F_2y, F_2z ], 
        ... ]
    displacement: An atomic displacement in Cartesian coordiante
      [ d_x, d_y, d_z ]
    """
    
    def __init__(self, atom_number, displacement, forces,
                 is_translational_invariance=False):
        self.atom_number = atom_number
        self.displacement = displacement
        self.forces = np.array(forces)
        if is_translational_invariance:
            self.set_translational_invariance()

    def get_atom_number(self):
        return self.atom_number

    def get_displacement(self):
        return self.displacement

    def get_forces(self):
        return self.forces

    def set_translational_invariance(self):
        self.forces = (self.forces - 
                       np.sum(self.forces, axis=0) / self.forces.shape[0])
        



#!/usr/bin/env python

import yaml
import numpy as np

data = yaml.load(open("qpoints.yaml"))
dynmat = []
dynmat_data = data['phonon'][0]['dynamical_matrix']
for row in dynmat_data:
    vals = np.reshape(row, (-1, 2))
    dynmat.append(vals[:, 0] + vals[:, 1] * 1j)
dynmat = np.array(dynmat)

eigvals, eigvecs, = np.linalg.eigh(dynmat)
frequencies = np.sqrt(np.abs(eigvals.real)) * np.sign(eigvals.real)
conversion_factor_to_THz = 15.633302
print frequencies * conversion_factor_to_THz





# Copyright (C) 2016 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from optparse import OptionParser

def get_parser():
    parser = OptionParser()
    parser.set_defaults(
        abinit_mode=False,
        anime=None,
        band_indices=None,
        band_labels=None,
        band_paths=None,
        band_points=None,
        cell_filename=None,
        cutoff_frequency=None,
        cutoff_radius=None,
        displacement_distance=None,
        dynamical_matrix_decimals=None,
        elk_mode=False,
        siesta_mode=False,
        fc_symmetry=None,
        fc_computation_algorithm=None,
        fc_spg_symmetry=False,
        fits_debye_model=False,
        force_constants_decimals=None,
        force_constants_mode=False,
        force_sets_mode=False,
        force_sets_zero_mode=False,
        frequency_conversion_factor=None,
        gv_delta_q=None,
        is_band_connection=False,
        is_check_symmetry=False,
        is_displacement=False,
        is_dos_mode=False,
        is_eigenvectors=False,
        is_gamma_center=False,
        is_graph_plot=False,
        is_graph_save=False,
        is_group_velocity=False,
        is_hdf5=False,
        is_legend=False,
        is_little_cogroup=False,
        is_nac=False,
        is_nodiag=False,
        is_nomeshsym=False,
        is_nosym=False,
        is_plusminus_displacements=False,
        is_read_force_constants=False,
        is_tetrahedron_method=False,
        is_thermal_displacements=False,
        is_thermal_displacement_matrices=False,
        is_thermal_properties=False,
        is_projected_thermal_properties=False,
        is_trigonal_displacements=False,
        is_wien2k_p1=False,
        irreps_qpoint=None,
        lapack_solver=False,
        loglevel=None,
        masses=None,
        mesh_numbers=None,
        modulation=None,
        primitive_axis=None,
        projection_direction=None,
        pwscf_mode=False,
        qpoints=None,
        quiet=False,
        q_direction=None,
        show_irreps=False,
        sigma=None,
        supercell_dimension=None,
        symprec=1e-5,
        tmax=None,
        tmin=None,
        tstep=None,
        verbose=False,
        wien2k_mode=False,
        write_dynamical_matrices=False,
        write_mesh=True,
        yaml_mode=False)
    
    parser.add_option(
        "--abinit", dest="abinit_mode",
        action="store_true", help="Invoke Abinit mode")
    parser.add_option(
        "--amplitude", dest="displacement_distance", type="float",
        help="Distance of displacements")
    parser.add_option(
        "--anime", dest="anime", action="store", type="string",
        help="Same as ANIME tag")
    parser.add_option(
        "--band", dest="band_paths", action="store", type="string",
        help="Same behavior as BAND tag")
    parser.add_option(
        "--band_connection", dest="is_band_connection",
        action="store_true", help="Treat band crossings")
    parser.add_option(
        "--band_labels", dest="band_labels", action="store", type="string",
        help="Show labels at band segments")
    parser.add_option(
        "--band_points", dest="band_points", type="int",
        help=("Number of points calculated on a band segment in "
              "the band structure mode"))
    parser.add_option(
        "--bi", "--band_indices", dest="band_indices", type="string",
        help=("Band indices to be included to calcualte thermal "
              "properties"))
    parser.add_option(
        "-c", "--cell", dest="cell_filename", action="store", type="string",
        help="Read unit cell", metavar="FILE")
    parser.add_option(
        "--cutoff_freq", "--cutoff_frequency",
        dest="cutoff_frequency", type="float",
        help=("Thermal properties are not calculated below this "
              "cutoff frequency."))
    parser.add_option(
        "--cutoff_radius", dest="cutoff_radius", type="float",
        help="Out of cutoff radius, force constants are set zero.")
    parser.add_option(
        "-d", "--displacement", dest="is_displacement", action="store_true",
        help="Create supercells with displacements")
    parser.add_option(
        "--dim", dest="supercell_dimension", action="store", type="string",
        help="Same behavior as DIM tag")
    parser.add_option(
        "--dm_decimals", dest="dynamical_matrix_decimals",
        type="int", help="Decimals of values of decimals")
    parser.add_option(
        "--dos", dest="is_dos_mode", action="store_true",
        help="Calculate (P)DOS")
    parser.add_option(
        "--eigvecs", "--eigenvectors", dest="is_eigenvectors",
        action="store_true",
        help="Output eigenvectors")
    parser.add_option(
        "--elk", dest="elk_mode", action="store_true",
        help="Invoke elk mode")
    parser.add_option(
        "-f", "--force_sets", dest="force_sets_mode", action="store_true",
        help="Create FORCE_SETS")
    parser.add_option(
        "--factor", dest="frequency_conversion_factor", type="float",
        help="Conversion factor to favorite frequency unit")
    parser.add_option(
        "--fc", "--force_constants", dest="force_constants_mode",
        action="store_true",
        help=("Create FORCE_CONSTANTS from vaspurn.xml. "
              "vasprun.xml has to be passed as argument."))
    parser.add_option(
        "--fc_decimals", dest="force_constants_decimals",
        type="int", help="Decimals of values of force constants")
    parser.add_option(
        "--fc_computation_algorithm", dest="fc_computation_algorithm",
        action="store", type="string",
        help="Switch computation algorithm of force constants")
    parser.add_option(
        "--fc_spg_symmetry", dest="fc_spg_symmetry", action="store_true",
        help="Enforce space group symmetry to force constants")
    parser.add_option(
        "--fc_symmetry", dest="fc_symmetry", type="int",
        help="Symmetrize force constants")
    parser.add_option(
        "--fits_debye_model", dest="fits_debye_model", action="store_true",
        help="Fits total DOS to a Debye model")
    parser.add_option(
        "--gc", "--gamma_center", dest="is_gamma_center", action="store_true",
        help="Set mesh as Gamma center")
    parser.add_option(
        "--gv", "--group_velocity", dest="is_group_velocity",
        action="store_true",
        help="Calculate group velocities at q-points")
    parser.add_option(
        "--gv_delta_q", dest="gv_delta_q", type="float",
        help="Delta-q distance used for group velocity calculation")
    parser.add_option(
        "--hdf5", dest="is_hdf5", action="store_true",
        help="Use hdf5 for force constants")
    parser.add_option(
        "--irreps", "--irreps_qpoint", dest="irreps_qpoint",
        action="store", type="string",
        help="A q-point where characters of irreps are calculated")
    parser.add_option(
        "--lapack_solver", dest="lapack_solver", action="store_true",
        help=("Use Lapack via Lapacke for solving phonons. This "
              "option can be used only when phonopy is compiled "
              "specially."))
    parser.add_option(
        "--legend", dest="is_legend", action="store_true",
        help="Legend of plots is shown in thermal displacements")
    parser.add_option(
        "--lcg", "--little_cogroup", dest="is_little_cogroup",
        action="store_true",
        help=("Show irreps of little co-group (or point-group of "
              "wave vector q) instead of little group"))
    parser.add_option(
        "--loglevel", dest="loglevel", type="int",
        help="Log level")
    parser.add_option(
        "--mass", dest="masses", action="store", type="string",
        help="Same as MASS tag")
    parser.add_option(
        "--modulation", dest="modulation", action="store", type="string",
        help="Same as MODULATION tag")
    parser.add_option(
        "--mp", "--mesh", dest="mesh_numbers", action="store", type="string",
        help="Same behavior as MP tag")
    parser.add_option(
        "--nac", dest="is_nac", action="store_true",
        help="Non-analytical term correction")
    parser.add_option(
        "--nodiag", dest="is_nodiag", action="store_true",
        help="Set displacements parallel to axes")
    parser.add_option(
        "--nomeshsym", dest="is_nomeshsym", action="store_true",
        help="Symmetry is not imposed for mesh sampling.")
    parser.add_option(
        "--nowritemesh", dest="write_mesh", action="store_false",
        help="Do not write mesh.yaml or mesh.hdf5")
    parser.add_option(
        "--nosym", dest="is_nosym", action="store_true",
        help="Symmetry is not imposed.")
    parser.add_option(
        "-p", "--plot", dest="is_graph_plot", action="store_true",
        help="Plot data")
    parser.add_option(
        "--pa", "--primitive_axis", dest="primitive_axis",
        action="store", type="string",
        help="Same as PRIMITIVE_AXIS tag")
    parser.add_option(
        "--pd", "--projection_direction", dest="projection_direction",
        action="store", type="string",
        help="Same as PROJECTION_DIRECTION tag")
    parser.add_option(
        "--pdos", dest="pdos", action="store", type="string",
        help="Same as PDOS tag")
    parser.add_option(
        "--pm", dest="is_plusminus_displacements", action="store_true",
        help="Set plus minus displacements")
    parser.add_option(
        "--pt", "--projected_thermal_property",
        dest="is_projected_thermal_properties", action="store_true",
        help="Output projected thermal properties")
    parser.add_option(
        "--pwscf", dest="pwscf_mode",
        action="store_true", help="Invoke Pwscf mode")
    parser.add_option(
        "--siesta", dest="siesta_mode",
        action="store_true", help="Invoke Siesta mode")
    parser.add_option(
        "-q", "--quiet", dest="quiet", action="store_true",
        help="Print out smallest information")
    parser.add_option(
        "--readfc", dest="is_read_force_constants", action="store_true",
        help="Read FORCE_CONSTANTS")
    parser.add_option(
        "-s", "--save", dest="is_graph_save", action="store_true",
        help="Save plot data in pdf")
    parser.add_option(
        "--show_irreps", dest="show_irreps", action="store_true",
        help="Show IR-Reps along with characters")
    parser.add_option(
        "--sigma", dest="sigma", type="string",
        help="Smearing width for DOS")
    parser.add_option(
        "--symmetry", dest="is_check_symmetry", action="store_true",
        help="Check crystal symmetry")
    parser.add_option(
        "-t", "--thermal_property", dest="is_thermal_properties",
        action="store_true",
        help="Output thermal properties")
    parser.add_option(
        "--td", "--thermal_displacements",
        dest="is_thermal_displacements", action="store_true",
        help="Output thermal displacements")
    parser.add_option(
        "--tdm", "--thermal_displacement_matrix",
        dest="is_thermal_displacement_matrices", action="store_true",
        help="Output thermal displacement matrices")
    parser.add_option(
        "--thm", "--tetrahedron_method",
        dest="is_tetrahedron_method", action="store_true",
        help="Use tetrahedron method for DOS/PDOS")
    parser.add_option(
        "--tmax", dest="tmax", type="string",
        help="Maximum calculated temperature")
    parser.add_option(
        "--qpoints", dest="qpoints", type="string",
        help="Calculate at specified q-points")
    parser.add_option(
        "--q_direction", dest="q_direction", type="string",
        help=("Direction of q-vector perturbation used for NAC at "
              "q->0, and group velocity for degenerate phonon "
              "mode in q-points mode"))
    parser.add_option(
        "--tmin", dest="tmin", type="string",
        help="Minimum calculated temperature")
    parser.add_option(
        "--trigonal", dest="is_trigonal_displacements", action="store_true",
        help="Set displacements of all trigonal axes ")
    parser.add_option(
        "--tstep", dest="tstep", type="string",
        help="Calculated temperature step")
    parser.add_option(
        "--tolerance", dest="symprec", type="float",
        help="Symmetry tolerance to search")
    parser.add_option(
        "-v", "--verbose", dest="verbose", action="store_true",
        help="Detailed information is shown.")
    parser.add_option(
        "--wien2k", dest="wien2k_mode",
        action="store_true", help="Invoke Wien2k mode")
    parser.add_option(
        "--wien2k_p1", dest="is_wien2k_p1", action="store_true",
        help="Assume Wien2k structs with displacements are P1")
    parser.add_option(
        "--writefc", dest="write_force_constants", action="store_true",
        help="Write FORCE_CONSTANTS")
    parser.add_option(
        "--writedm", dest="write_dynamical_matrices", action="store_true",
        help=("Write dynamical matrices. This has to be used "
              "with QPOINTS setting (or --qpoints)"))
    parser.add_option(
        "--yaml", dest="yaml_mode", action="store_true",
        help="Activate phonopy YAML mode")

    return parser



from phonopy import Phonopy
from phonopy.interface.vasp import read_vasp
from phonopy.file_IO import parse_FORCE_SETS
import numpy as np

def get_frequency(poscar_filename, force_sets_filename):
    bulk = read_vasp(poscar_filename)
    volume = bulk.get_volume()
    phonon = Phonopy(bulk, [[2, 0, 0], [0, 2, 0], [0, 0, 2]],
                     is_auto_displacements=False)
    force_sets = parse_FORCE_SETS(filename=force_sets_filename)
    phonon.set_force_sets(force_sets)
    phonon.set_post_process([[0, 0.5, 0.5], [0.5, 0, 0.5], [0.5, 0.5, 0]])
    return phonon.get_frequencies([0.5, 0.5, 0]), volume

frequencies = []
volumes = []
for i in range(-10, 6):
    poscar_filename = "POSCAR-%d" % i
    force_sets_filename = "FORCE_SETS-%d" % i
    fs, v = get_frequency(poscar_filename, force_sets_filename)
    frequencies.append(fs)
    volumes.append(v)

import matplotlib.pyplot as plt
for curve in np.array(frequencies).T:
    plt.plot(volumes, curve**2 * np.sign(curve))
    for v, f2 in zip(volumes, curve ** 2 * np.sign(curve)):
         print v, f2
    print
    print
plt.show()



# Copyright (C) 2015 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

from __future__ import print_function
import sys
import numpy as np

try:
    import yaml
except ImportError:
    print("You need to install python-yaml.")
    sys.exit(1)

try:
    from yaml import CLoader as Loader
    from yaml import CDumper as Dumper
except ImportError:
    from yaml import Loader, Dumper

from phonopy.structure.atoms import Atoms, PhonopyAtoms

class PhonopyYaml:
    def __init__(self, filename):
        with open(filename) as infile :
            self._data = yaml.load( infile, Loader=Loader)
        self._lattice = self._data['lattice']

        self._points = None
        self._symbols = None
        self._masses = None

        points = []
        symbols = []
        masses = []
        for x in self._data['points']:
            if 'coordinates' in x:
                points.append(x['coordinates'])
            if 'symbol' in x:
                symbols.append(x['symbol'])
            if 'mass' in x:
                masses.append(x['mass'])
        if points:
            self._points = points
        if symbols:
            self._symbols = symbols
        if masses:
            self._masses = masses

    def get_atoms(self):
        return Atoms(symbols=self._symbols,
                     cell=self._lattice,
                     masses=self._masses,
                     scaled_positions=self._points)

    def __str__(self):
        return str(PhonopyAtoms(atoms=self.get_atoms()))



# Copyright (C) 2014 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import os
from phonopy.file_IO import parse_disp_yaml, write_FORCE_SETS

def read_crystal_structure(filename=None,
                           interface_mode='vasp',
                           chemical_symbols=None,
                           yaml_mode=False):
    if filename is None:
        unitcell_filename = get_default_cell_filename(interface_mode, yaml_mode)
    else:
        unitcell_filename = filename

    if not os.path.exists(unitcell_filename):
        if filename is None:
            return None, (unitcell_filename + " (default file name)",)
        else:
            return None, (unitcell_filename,)
    
    if yaml_mode:
        from phonopy.interface.phonopy_yaml import PhonopyYaml
        unitcell = PhonopyYaml(unitcell_filename).get_atoms()
        return unitcell, (unitcell_filename,)
        
    if interface_mode == 'vasp':
        from phonopy.interface.vasp import read_vasp
        if chemical_symbols is None:
            unitcell = read_vasp(unitcell_filename)
        else:
            unitcell = read_vasp(unitcell_filename, symbols=chemical_symbols)
        return unitcell, (unitcell_filename,)
        
    if interface_mode == 'abinit':
        from phonopy.interface.abinit import read_abinit
        unitcell = read_abinit(unitcell_filename)
        return unitcell, (unitcell_filename,)
        
    if interface_mode == 'pwscf':
        from phonopy.interface.pwscf import read_pwscf
        unitcell, pp_filenames = read_pwscf(unitcell_filename)
        return unitcell, (unitcell_filename, pp_filenames)
        
    if interface_mode == 'wien2k':
        from phonopy.interface.wien2k import parse_wien2k_struct
        unitcell, npts, r0s, rmts = parse_wien2k_struct(unitcell_filename)
        return unitcell, (unitcell_filename, npts, r0s, rmts)

    if interface_mode == 'elk':
        from phonopy.interface.elk import read_elk
        unitcell, sp_filenames = read_elk(unitcell_filename)
        return unitcell, (unitcell_filename, sp_filenames)

    if interface_mode == 'siesta':
        from phonopy.interface.siesta import read_siesta
        unitcell, atypes = read_siesta(unitcell_filename)
        return unitcell, (unitcell_filename, atypes)

def get_default_cell_filename(interface_mode, yaml_mode):
    if yaml_mode:
        return "POSCAR.yaml"
    if interface_mode == 'vasp':
        return "POSCAR"
    if interface_mode == 'abinit':
        return "unitcell.in"
    if interface_mode == 'pwscf':
        return "unitcell.in"
    if interface_mode == 'wien2k':
        return "case.struct"
    if interface_mode == 'elk':
        return "elk.in"
    if interface_mode == 'siesta':
        return "input.fdf" 

def create_FORCE_SETS(interface_mode,
                      force_filenames,
                      symprec,
                      is_wien2k_p1=False,
                      log_level=0):
    if (interface_mode == 'vasp' or
        interface_mode == 'abinit' or
        interface_mode == 'elk' or
        interface_mode == 'pwscf' or
        interface_mode == 'siesta'):
        displacements = parse_disp_yaml(filename='disp.yaml')
        num_atoms = displacements['natom']
        if len(displacements['first_atoms']) != len(force_filenames):
            print('')
            print("Number of files to be read don't match "
                  "to number of displacements in disp.yaml.")
            return 1
        force_sets = get_force_sets(interface_mode, num_atoms, force_filenames)

    elif interface_mode == 'wien2k':
        displacements, supercell = parse_disp_yaml(filename='disp.yaml',
                                                   return_cell=True)
        if len(displacements['first_atoms']) != len(force_filenames):
            print('')
            print("Number of files to be read don't match "
                  "to number of displacements in disp.yaml.")
            return 1
        from phonopy.interface.wien2k import parse_set_of_forces
        force_sets = parse_set_of_forces(
            displacements,
            force_filenames,
            supercell,
            disp_keyword='first_atoms',
            is_distribute=(not is_wien2k_p1),
            symprec=symprec)
    else:
        force_sets = []

    if force_sets:
        for forces, disp in zip(force_sets, displacements['first_atoms']):
            disp['forces'] = forces
        write_FORCE_SETS(displacements, filename='FORCE_SETS')
        
    if log_level > 0:
        if force_sets:
            print("FORCE_SETS has been created.")
        else:
            print("FORCE_SETS could not be created.")

    return 0
            
def get_force_sets(interface_mode, num_atoms, force_filenames):
    if interface_mode == 'vasp':
        from phonopy.interface.vasp import parse_set_of_forces
        force_sets = parse_set_of_forces(num_atoms, force_filenames)
    elif interface_mode == 'abinit':
        from phonopy.interface.abinit import parse_set_of_forces
        force_sets = parse_set_of_forces(num_atoms, force_filenames)
    elif interface_mode == 'pwscf':
        from phonopy.interface.pwscf import parse_set_of_forces
        force_sets = parse_set_of_forces(num_atoms, force_filenames)
    elif interface_mode == 'elk':
        from phonopy.interface.elk import parse_set_of_forces
        force_sets = parse_set_of_forces(num_atoms, force_filenames)
    elif interface_mode == 'siesta':
        from phonopy.interface.siesta import parse_set_of_forces
        force_sets = parse_set_of_forces(num_atoms, force_filenames)

    return force_sets



# Copyright (C) 2011 Atsushi Togo
# All rights reserved.
#
# This file is part of phonopy.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
# * Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer.
#
# * Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in
#   the documentation and/or other materials provided with the
#   distribution.
#
# * Neither the name of the phonopy project nor the names of its
#   contributors may be used to endorse or promote products derived
#   from this software without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import numpy as np
import phonopy.structure.spglib as spg
from phonopy.structure.symmetry import Symmetry, find_primitive, get_pointgroup
from phonopy.structure.cells import get_primitive, print_cell, get_supercell
from phonopy.interface.vasp import write_vasp
from phonopy.structure.atoms import Atoms

def get_symmetry_yaml(cell, symmetry, phonopy_version=None):
    rotations = symmetry.get_symmetry_operations()['rotations']
    translations = symmetry.get_symmetry_operations()['translations']
    
    atom_sets = symmetry.get_map_atoms()
    independent_atoms = symmetry.get_independent_atoms()
    wyckoffs = symmetry.get_Wyckoff_letters()

    yaml = ""

    if phonopy_version is not None:
        yaml += "phonopy_version: %s\n" % phonopy_version

    if cell.get_magnetic_moments() is None:
        yaml += "space_group_type: " + symmetry.get_international_table() + "\n"
    yaml += "point_group_type: " + symmetry.get_pointgroup() + "\n"
    yaml += "space_group_operations:\n"
    for i, (r, t) in enumerate(zip(rotations, translations)):
        yaml += "- rotation: # %d\n" % (i+1)
        for vec in r:
            yaml += "  - [%2d, %2d ,%2d]\n" % tuple(vec)
        yaml += "  translation: ["
        for j, x in enumerate(t):
            if abs(x - np.rint(x)) < 1e-5:
                yaml += " 0.00000"
            else:
                yaml += "%8.5f" % x
            if j < 2:
                yaml += ", "
            else:
                yaml += "]\n"
    yaml += "atom_mapping:\n"
    for i, atom_num in enumerate(atom_sets):
        yaml += "  %d: %d\n" % (i+1, atom_num+1)
    yaml += "site_symmetries:\n"
    for i in independent_atoms:
        sitesym = symmetry.get_site_symmetry(i)
        yaml += "- atom: %d\n" % (i+1)

        if cell.get_magnetic_moments() is None:
            yaml += "  Wyckoff: %s\n" % (wyckoffs[i])
        site_pointgroup = get_pointgroup(sitesym)
        yaml += "  site_point_group: %s\n" % (site_pointgroup[0])
        yaml += "  orientation:\n"
        for v in site_pointgroup[1]:
            yaml += "  - [%2d, %2d, %2d]\n" % tuple(v)

        yaml += "  rotations:\n"
        for j, r in enumerate(sitesym):
            yaml += "  - # %d\n" % (j+1)
            for vec in r:
                yaml += "    - [%2d, %2d, %2d]\n" % tuple(vec)

    return yaml

def check_symmetry(input_cell,
                   primitive_axis=None,
                   symprec=1e-5,
                   phonopy_version=None):
    if primitive_axis is None:
        cell = get_primitive(input_cell, np.eye(3), symprec=symprec)
    else:
        cell = get_primitive(input_cell, primitive_axis, symprec=symprec)
    symmetry = Symmetry(cell, symprec)
    print(get_symmetry_yaml(cell, symmetry, phonopy_version))

    if input_cell.get_magnetic_moments() is None:
        primitive = find_primitive(cell, symprec)
        if primitive is not None:
            print("# Primitive cell was found. It is written into PPOSCAR.")
            write_vasp('PPOSCAR', primitive)
            
            # Overwrite symmetry and cell
            symmetry = Symmetry(primitive, symprec)
            cell = primitive
    
        bravais_lattice, bravais_pos, bravais_numbers = \
            spg.refine_cell(cell, symprec)
        bravais = Atoms(numbers=bravais_numbers,
                        scaled_positions=bravais_pos,
                        cell=bravais_lattice,
                        pbc=True)
        print("# Bravais lattice is written into BPOSCAR.")
        write_vasp('BPOSCAR', bravais)



